#! /usr/bin/python
# coding=utf-8
# -*- encoding: utf-8 -*-
#
__author__='atareao'
__date__ ='$06-jun-2010 12:34:44$'
#
# <one line to give the program's name and a brief idea of what it does.>
#
# Copyright (C) 2010 Lorenzo Carbonell
# lorenzo.carbonell.cerezo@gmail.com
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#
#
import locale
import re
import gettext
import dbus
import subprocess
import htmlentitydefs, re

locale.setlocale(locale.LC_ALL, '')
gettext.bindtextdomain('unity-lens-translator', '/usr/share/locale-langpack')
gettext.textdomain('unity-lens-translator')
_ = gettext.gettext


TRANSLATIONS = {}
TRANSLATIONS['ca-en']=_('Catalan to English')
TRANSLATIONS['ca-eo']=_('Catalan to Esperanto')
TRANSLATIONS['ca-es']=_('Catalan to Spanish')
TRANSLATIONS['ca-fr']=_('Catalan to French')
TRANSLATIONS['ca-oc']=_('Catalan to Occitan')
TRANSLATIONS['ca-pt']=_('Catalan to Potuguese')
TRANSLATIONS['en-ca']=_('English to Catalan')
TRANSLATIONS['en-es']=_('English to Spanish')
TRANSLATIONS['es-ca']=_('Spanish to Catalan')
TRANSLATIONS['es-en']=_('Spanish to English')
TRANSLATIONS['es-eo']=_('Spanish to Esperanto')
TRANSLATIONS['es-fr']=_('Spanish to French')
TRANSLATIONS['es-gl']=_('Spanish to Galician')
TRANSLATIONS['es-oc']=_('Spanish to Occitan')
TRANSLATIONS['es-pt']=_('Spanish to Portuguese')
TRANSLATIONS['eu-es']=_('Euskera to Spanish')
TRANSLATIONS['fr-ca']=_('French to Catalan')
TRANSLATIONS['fr-es']=_('French to Spanish')
TRANSLATIONS['gl-es']=_('Galician to Spanish')
TRANSLATIONS['gl-pt']=_('Galician to Portuguese')
TRANSLATIONS['oc-ca']=_('Occitan to Catalan')
TRANSLATIONS['oc-es']=_('Occitan to Spanish')
TRANSLATIONS['pt-ca']=_('Portuguese to Catalan')
TRANSLATIONS['pt-es']=_('Portuguese to Spanish')
TRANSLATIONS['pt-gl']=_('Portuguese to Galician')
TRANSLATIONS['ro-es']=_('Romanian to Spanish')

def get_translations():
	translations = []
	bus = dbus.SessionBus() # Create a new session bus
	info = dbus.Interface(bus.get_object('org.apertium.info', '/'), 'org.apertium.Info'); 
	for cad in info.modes():
		if cad in TRANSLATIONS.keys():
			translations.append(cad)
	return translations

def get_key_translation():
	ans = []
	for translation in get_translations():
		ans.append([TRANSLATIONS[translation],str(translation)])
	ans.sort()
	return ans
	

def translate_subpart(string, lang_direction):
	"""Simple translate for just a certin string"""

	for codes in lang_direction.split("/"):
		translater = subprocess.Popen(['apertium', '-u', '-f', 'html', codes], stdin=subprocess.PIPE, stdout=subprocess.PIPE)
		translater.stdin.write(string.encode("utf8")+"\n")
		string, _ = translater.communicate()
		string = string[:-1].decode("utf8")

	return string

def translate2(string, lang_direction):
	bus = dbus.SessionBus() # Create a new session bus
	translator = dbus.Interface(bus.get_object('org.apertium.mode', '/'), 'org.apertium.Translate'); 
	utf = dbus.UTF8String(string)
	ans = translator.translate(lang_direction, {}, utf)
	return ans

	
def translate(string, lang_direction):
	if type(string) == str:
		string = string.decode('utf-8')
	"""Takes a string that is to be translated and returns the translated string, doesn't translate the %(format)s parts, they must remain the same text as the msgid"""
	# simple format chars like %s can be 'translated' ok, they just pass through unaffected
	named_format_regex = re.compile(r"%\([^\)]+?\)[sd]", re.VERBOSE)
	matches = named_format_regex.findall(string)
	new = None

	if len(matches) == 0:
		# There are no format specifiers in this string, so just do a straight translation

		# this fails if we've missed a format specifier
		assert "%(" not in string, "This code has a bug and has missed a format specifier in this string: "+repr(string)

		new = translate_subpart(string, lang_direction)

	else:
		
		# The format specifiers ("... %(name)s ... ") might get translated into
		# the target language, they should not. So replace them with some text
		# that is highly unlikely to be translated
		matches = set(matches)
		match_replace = [(x, '__MTCH%04d__' % i) for i, x in enumerate(matches)]

		intermediate_string = string

		for old, new in match_replace:
			intermediate_string = intermediate_string.replace(old, new)

		translated_string = translate_subpart(intermediate_string, lang_direction)

		
		for old, new in match_replace:
			translated_string = translated_string.replace(new, old)

		new = translated_string
	
	# sometimes the start string "\n", might have gotten changed to ". \n"
	# compilemessages doesn't like this, so strip them out
	while string.startswith("\n") and len(new) > 0 and new.startswith(". \n"):
		new = new[2:]
	new = re.sub('&([^;]+);', lambda m: unichr(htmlentitydefs.name2codepoint[m.group(1)]), new)
	return  new.encode('utf-8')


if __name__ == '__main__':
	print 'camión'.decode('utf-8')
	value = dbus.UTF8String('camión')
	print type(value)
	print value
	print translate('truck','en-es')
	print get_key_translation()
	'''
	for translation in get_key_translation():
		print translation
	print type(u'Este es mi camión')
	print translate2(u'Este es mi camión','es-en')
	ans =  translate('This is my truck','en-es')
	print ans
	'''
	'''
	import babel
	for translation in get_translations():
		de,a=translation.split('-')
		lan = locale.getlocale()[0]
		de_name = babel.Locale(de).get_display_name(lan).capitalize()
		a_name = babel.Locale(a).get_display_name(lan).capitalize()
		print '%s -> %s'%(de_name,a_name)
	'''
